#pragma once

#include <optional>
#include <deque>

#include <QMenu>
#include <QWidget>
#include <QGroupBox>

#include <SimoxUtility/color/ColorMap.h>

#include <ArmarXCore/core/logging/Logging.h>

#include <RobotAPI/libraries/aron/core/type/variant/forward_declarations.h>
#include <RobotAPI/libraries/aron/core/Path.h>

#include <RobotAPI/libraries/armem/core/wm/memory_definitions.h>


class QGroupBox;
class QLabel;
class QSplitter;
class QTreeWidget;
class QTreeWidgetItem;


namespace armarx::armem::gui::instance
{
    class ImageView;
    class MemoryIDTreeWidgetItem;
    class WidgetsWithToolbar;


    class InstanceView : public QWidget, public armarx::Logging
    {
        Q_OBJECT
        using This = InstanceView;


    public:

        InstanceView();

        void setStatusLabel(QLabel* statusLabel);
        void setUseTypeInfo(bool enable);

        void update(const MemoryID& id, const wm::Memory& memory);
        void update(const wm::EntityInstance& instance, aron::type::ObjectPtr aronType = nullptr);
        void update();

        void addInstanceView(const wm::EntityInstance& instance, aron::type::ObjectPtr aronType = nullptr);


    signals:

        void updated();
        void instanceSelected(const MemoryID& id);
        void memoryIdResolutionRequested(const MemoryID& id);
        void actionsMenuRequested(const MemoryID& memoryID, QWidget* parent,
                const QPoint& pos, QMenu* menu);


    private slots:

        void prepareTreeContextMenu(const QPoint& pos);

        void showImageView(const aron::Path& elementPath);
        void removeImageView();


    private:

        void updateInstanceID(const MemoryID& id);
        void updateData(const aron::data::DictPtr& data, aron::type::ObjectPtr aronType = nullptr);
        void updateMetaData(const wm::EntityInstanceMetadata& metadata);
        void updateImageView(const aron::data::DictPtr& data);

        void showErrorMessage(const std::string& message);

        std::optional<aron::Path> getElementPath(const QTreeWidgetItem* item) const;
        std::optional<MemoryID> getElementMemoryID(const aron::Path& elementPath);

        QAction* makeActionResolveMemoryID(const MemoryID& id);
        QAction* makeActionCopyMemoryID(const MemoryID& id);
        std::vector<QAction*> makeActionsCopyDataToClipboard();
        std::vector<QAction*> makeActionsCopyDataToClipboard(const aron::Path& path);
        std::vector<QAction*> makeCopyActions(const aron::data::VariantPtr& element);


    private:

        enum class Columns
        {
            KEY = 0,
            VALUE = 1,
            TYPE = 2,
        };

        std::optional<wm::EntityInstance> currentInstance;
        aron::type::ObjectPtr currentAronType = nullptr;
        bool useTypeInfo = true;

        QSplitter* splitter;

        QTreeWidget* tree;
        MemoryIDTreeWidgetItem* treeItemInstanceID;
        QTreeWidgetItem* treeItemMetadata;
        QTreeWidgetItem* treeItemData;


        class ImageView : public QGroupBox
        {
        public:
            ImageView();

            QImage convertDepth32ToRGB32(const aron::data::NDArray& aron);

            instance::ImageView* view;
            aron::Path elementPath;

            WidgetsWithToolbar* toolbar;


            struct Limits
            {
                float min = std::numeric_limits<float>::max();
                float max = -std::numeric_limits<float>::max();
            };

            /// Color map to visualize depth images.
            simox::ColorMap cmap;
            /// History over first n extremal depth values used to calibrate the colormap.
            std::deque<Limits> limitsHistory;
            /// In this context, n.
            const size_t limitsHistoryMaxSize;
        };
        ImageView* imageView = nullptr;

        QLabel* statusLabel = nullptr;

    };

}

namespace armarx::armem::gui
{
    using InstanceView = instance::InstanceView;
}
